# ASM stubs

This directory contains:

- A header that can be used on different compilers to include the ASM stubs used during the auto-record.
- A Rust module that can be used in your crates to gain auto-record capabilities.
- A Python script that can be used on guests that have python3 to start and stop auto-record.

Functions defined in `asm_stubs.h` are:
 * `host_start_record(void)`
 * `host_stop_record(void)`
 * `host_commit_record(void)`
 * `host_abort_record(const char* reason)`

## C/C++

The `asm_stubs.h` file can be put next to `main.c` and used like this:

`main.c`:
```c
#include <stdio.h>
#include "asm_stubs.h"

int main() {
  printf("Starting auto-record\n");
  host_start_record();

  printf("Hello there!\n");

  host_stop_record();
  printf("Auto-record stopped, committing\n");
  host_commit_record();
}
```

### Linux

You just need to include `asm_stubs.h` (you don't need `asm_stubs.asm`).

### Windows

Because inline ASM is disabled in Visual Studio since the 2017 version, you need both files. You need to include `asm_stubs.h` and also compile `asm_stubs.asm` in your project.

**Warning: You need to enable the compilation of assembly using `masm` on your project**


## Rust

The `asm_stubs.rs` file can be put next to `main.rs` and used like this:

`main.rs`:
```rust
mod asm_stubs;

fn main() {
    println!("Starting auto-record");
    asm_stubs::autorecord::start_record();

    println!("Hello there!");

    asm_stubs::autorecord::stop_record();
    println!("Auto-record stopped, committing");
    asm_stubs::autorecord::commit_record();
}
```


## Python

The `asm_stubs.py` file can be put next to `main.py` and used like this:

`main.py`:
```python
#!/usr/bin/python3

import asm_stubs


print("Starting auto-record")
asm_stubs.start_record()

print("Hello there!")

asm_stubs.stop_record()
print("Auto-record stopped, committing")
asm_stubs.commit_record()
```